
/* BEGIN COPYRIGHT NOTICE

Copyright (c) 1992 by John Sarraille and Peter DiFalco
(john@ishi.csustan.edu)

Copyright (c) 1998 by John Sarraille
(john@ishi.csustan.edu)

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that copyright notice and this permission
notice appear in supporting documentation.

The algorithm used in this program was inspired by the paper
entitled "A Fast Algorithm To Determine Fractal Dimensions By
Box Counting", which was written by Liebovitch and Toth, and
which appeared in the journal "Physics Letters A", volume 141,
pp 386-390, (1989).

This program is not warranted: use at your own risk.

END COPYRIGHT NOTICE */


/*
   FD3 PROGRAM FILE

Filename:  getcom.c

OUTLINE OF CALLS:

GetCommand +
  InitCommand +
  GetInfileName +
  Do1stOptStr +
    GetOptions +
      CkSet+
    TakesArgs +
    GetOptArgs +
      mOptArgs
  MoreOptStr+
    GetOptions +
      CkSet+
    TakesArgs +
    GetOptArgs +
      mOptArgs
  ConsistCheck +
PrintCommand+
*/

#include "fd.h"

/* 
    THE FUNCTIONS IN THIS FILE: 
*/
int   GetCommand      () ;
void  InitCommand     () ;
int   GetInfileName   () ;
int   Do1stOptStr     () ;
int   GetOptions      () ;
int   CkSet           () ;
int   TakesArgs       () ;
int   GetOptArgs      () ;
int   MoreOptStr      () ;
int   ConsistCheck    () ;
void  PrintCommand    () ;

/*   
      THE FUNCTIONS NOT IN THIS FILE,
      BUT CALLED FROM HERE:
*/
extern int      mOptArgs    () ;     /*  in optm.c    */
extern int      pOptArgs    () ;     /*  in optp.c    */
/* ######################################################### */
/*                       GetCommand                          */
/* ######################################################### */
/*
     INITIALIZE THE STRUCT USED TO REPRESENT THE COMMAND AND
     INTERPRET THE COMMAND.  THE STRATEGY WE ARE USING TO PARSE
     THE COMMAND SHOULD MAKE IT RELATIVELY EASY TO ADD MORE
     OPTIONS TO THE PROGRAM LATER.  EVEN OPTIONS THAT TAKE
     PARAMETERS WILL NOT BE HARD.
*/
int GetCommand (argc, argv, cPt, mPt )
       int argc ;
       char * argv[] ;
       commandType * cPt ;
       machineType * mPt ;
  {

    int syntaxOK, nChek ;
            /*
               THE NUMBER OF ARGUMENTS WE STILL NEED TO PROCESS
               IN ORDER TO GET THE SEMANTIC CONTENT.
            */
    int argsLeft = argc-1 ; 
            /*
               THE INDEX OF THE FIRST ARGUMENT THAT STILL NEEDS
               TO BE PROCESSED FOR SEMANTIC CONTENT.
            */
    int curArg = 1 ;

    InitCommand (cPt, mPt) ;
       /*
           A COMMAND CONSISTING ONLY OF THE PROGRAM NAME IS NOT
           ALLOWED.
       */
    if    (argsLeft == 0) syntaxOK = FALSE ;
    else  {   /* GET THE FILENAME PARAMETER OFF THE RIGHT END. */
            syntaxOK = GetInfileName (argv[argc-1], cPt) ;
            argsLeft = argsLeft - 1;
            if    ((syntaxOK) && (argsLeft != 0)) 
            {
                   /*
                       GET THE INITIAL OPTION STRING FROM
                       argv[1].  MAYBE WE WILL ALSO HAVE TO GET
                       ONE OR MORE ARGUMENTS THAT FOLLOW THE
                       OPTION STRING.
                   */
              nChek = Do1stOptStr(&argsLeft, &curArg, argv, cPt, mPt) ;
              syntaxOK = syntaxOK && nChek ;
            }
            while ((syntaxOK) && (argsLeft != 0)) 
            {
                   /*
                       GET THE REMAINING SINGLETON OPTION
                       STRINGS, TOGETHER WITH WHATEVER
                       ARGUMENTS APPLY TO THEM.
                   */
                nChek = MoreOptStr(&argsLeft, &curArg, argv, cPt, mPt) ;
                syntaxOK = syntaxOK && nChek ;
            }
            syntaxOK = syntaxOK && ConsistCheck (cPt) ;
          }
    return (syntaxOK) ;
  }
/* ######################################################### */
/*                        ConsistCheck                       */
/* ######################################################### */
/*
   THE TASK OF ConsistCheck IS TO SEE IF THE TOTALITY OF
   COMMAND LINE PARAMETERS ARE CONSISTENT.  FOR EXAMPLE, ARE
   THE ARGUMENTS TAKEN BY THE -p OPTION CONSITENT WITH THE
   NUMBER OF CELL SIZES CHOSEN BY THE USER WITH THE -m OPTION?

   AT THIS TIME, THERE IS LITTLE TO CHECK.

   WE USE THE "MAGIC NUMBER" 131 BELOW.  THIS WILL HAVE TO BE
   REPLACED WITH A FORMULA IF AND WHEN WE MAKE THE NUMBER OF
   CELL SIZES A PARAMETER OF THE -f OPTION.
*/

int ConsistCheck (cPt) 
       commandType * cPt ;
{
  int syntaxOK = TRUE ;

  if ( !(cPt->fine) && (cPt->placeBars)
          && ( ((cPt->numCellSizes)-1) < (cPt->fineMeshPoint) ))
  syntaxOK = FALSE ;

  if ( (cPt->fine) && (cPt->placeBars)
        && ( 131 < (cPt->fineMeshPoint) ) )
  syntaxOK = FALSE ;

  return(syntaxOK) ;
}
/* ######################################################### */
/*                        InitCommand                        */
/* ######################################################### */
void  InitCommand (commandPtr, mPt) 
         commandType *  commandPtr ;
         machineType *  mPt ;
   {
      strcpy (commandPtr->infileName,     "\"none\"")   ;
      (commandPtr->infileNameLength)   =  0     ;
      (commandPtr->checking)           =  FALSE ;
      (commandPtr->debugging)          =  FALSE ;
      (commandPtr->fine)               =  FALSE ;
      (commandPtr->mincover)           =  FALSE ;
      (commandPtr->scaleInfo)          =  FALSE ;
      (commandPtr->terse)              =  FALSE ;
      (commandPtr->verbose)            =  FALSE ;
      (commandPtr->placeBars)          =  FALSE ;
      (commandPtr->magnifier)          =  2.0   ;
      (commandPtr->numCellSizes)       = (mPt->ulongBitNum)+1;
      (commandPtr->saturationPoint)    =  0 ;
      (commandPtr->fineMeshPoint)      =  0 ;
      (commandPtr->invalidOption)      =  FALSE ;
      (commandPtr->repeatedOption)     =  FALSE ;
      (commandPtr->mutexOptions)       =  FALSE ;
      return ;
   }
/* ######################################################### */
/*                       GetInfileName                       */
/* ######################################################### */
int GetInfileName (namePtr, commandPtr)
       char *         namePtr ;
       commandType *  commandPtr ;
  {
        int syntaxOK = TRUE ;
        (commandPtr->infileNameLength) = strlen(namePtr) ;
        if    ( strlen(namePtr) <= MAXfNAMElen )
              strcpy (commandPtr->infileName, namePtr) ;
        else  syntaxOK = FALSE ;
        return (syntaxOK) ;
  }
/* ######################################################### */
/*                         Do1stOptStr                       */
/* ######################################################### */
/*
    GET THE FIRST STRING OF OPTIONS IN A COMMAND.  WE TRY TO
    DEAL WITH THE POSSIBILITY THAT THE COMMAND IS MALFORMED.
*/
int Do1stOptStr(argsLftPt, curArgPt, argv, cPt, mPt)
       int * argsLftPt ;
       int * curArgPt ;
       char * argv[] ;
       commandType * cPt ;
       machineType * mPt ;
{
  int syntaxOK, nChek ;

  syntaxOK = GetOptions (argv[(* curArgPt)], cPt) ;
  (* curArgPt) = (* curArgPt) + 1;
  (* argsLftPt) = (* argsLftPt) - 1;
  
  if (   syntaxOK
          && (strlen(argv[(* curArgPt)-1]) == 2)
          && (TakesArgs(argv[(* curArgPt)-1][1])) )
      {   
        nChek = GetOptArgs(argsLftPt, curArgPt, argv, cPt, mPt) ;
        syntaxOK = syntaxOK && nChek ;
      }
  return (syntaxOK) ;
}
/* ######################################################### */
/*                          MoreOptStr                       */
/* ######################################################### */
/*
    GET ONE OF THE SINGLETON STRINGS OF OPTIONS AFTER THE FIRST
    STRING OF OPTIONS IN A COMMAND.  WE TRY TO DEAL WITH THE
    POSSIBILITY THAT THE COMMAND IS MALFORMED.  THE RULES HERE
    ARE A LITTLE DIFFERENT FROM Do1stOptStr.  HERE THE OPTION
    STRING MUST CONTAIN JUST ONE OPTION, AND THE OPTION MUST BE
    ONE OF THOSE THAT *CAN* HAVE ONE OR MORE ARGUMENTS.
*/
int MoreOptStr(argsLftPt, curArgPt, argv, cPt, mPt) 
       int * argsLftPt ;
       int * curArgPt ;
       char * argv[] ;
       commandType * cPt ;
       machineType * mPt ;
{
  int syntaxOK, nChek ;

  syntaxOK = GetOptions(argv[(* curArgPt)], cPt) 
               && (strlen(argv[(* curArgPt)]) == 2)
               && (TakesArgs(argv[(* curArgPt)][1])) ;
  (* curArgPt) = (* curArgPt) + 1;
  (* argsLftPt) = (* argsLftPt) - 1;
  
  if (syntaxOK)
      {   
        nChek = GetOptArgs(argsLftPt, curArgPt, argv, cPt, mPt) ;
        syntaxOK = syntaxOK && nChek ;
      }
  return (syntaxOK) ;
}
/* ######################################################### */
/*                         GetOptions                        */
/* ######################################################### */
/*
    THIS FUNCTION ACCUMULATES THE OPTIONS CALLED FOR BY THE
    USER, AND TRIES TO PREVENT DUPLICATE OPTIONS AND
    COMBINATIONS OF OPTIONS THAT DON'T MAKE SENSE.  IT IS
    SUPPOSED TO BE POSSIBLE TO CALL THIS FUNCTION SEVERAL
    TIMES, ALLOWING RESULTS TO ACCUMULATE.  THIS FUNCTION DOES
    *NOT* PROCESS ANY OF THE ARGUMENTS THAT MIGHT CORRESPOND TO
    AN OPTION.  ALL THAT IS TAKEN CARE OF ELSEWHERE.
*/
int GetOptions (optStrPtr, cPt)
       char *         optStrPtr ;
       commandType *  cPt ;
{  int chPos, sOK, * rPT ;

   sOK = ((optStrPtr[0] == '-') && (strlen(optStrPtr) >= 2)) ;
   if (sOK)
   {
     rPT =  &(cPt->repeatedOption) ;
     for(chPos = 1; chPos < strlen(optStrPtr); chPos++)
     {  switch (optStrPtr[chPos])
        { case 'c': sOK = sOK && CkSet(&(cPt->checking), rPT);   break ;
          case 'd': sOK = sOK && CkSet(&(cPt->debugging), rPT);  break ;
          case 'f': sOK = sOK && CkSet(&(cPt->fine), rPT);       break ;
          case 'm': sOK = sOK && CkSet(&(cPt->mincover), rPT);   break ;
             /*
                 THIS OPTION IS REQUIRED TO OCCUR ALONE.
             */
          case 'p': sOK = sOK && CkSet(&(cPt->placeBars), rPT)
                              && (strlen(optStrPtr) == 2) ;      break ;
          case 's': sOK = sOK && CkSet(&(cPt->scaleInfo), rPT);  break ;
          case 't': sOK = sOK && CkSet(&(cPt->terse), rPT);      break ;
          case 'v': sOK = sOK && CkSet(&(cPt->verbose), rPT);    break ;
          default : sOK = FALSE ; (cPt->invalidOption) = TRUE ;  break ;
        }
     }
     if ((cPt->fine) && (cPt->mincover))
        {
          (cPt->mutexOptions) = TRUE ;
          sOK = FALSE ;
        }
   }
   return (sOK) ;
}
/* ######################################################### */
/*                            CkSet                          */
/* ######################################################### */
int CkSet (optFlagPtr, repeatFlagPtr)
      int * optFlagPtr, * repeatFlagPtr ;
{
        /* if the option flag is now false, then we may simply
           note that the option is now set, and return an
           indication that the syntax appears to be OK -- this
           is not a duplicate request for this option. */
    if ( (* optFlagPtr) == FALSE ) 
    {
       (* optFlagPtr) = TRUE ;
       return (TRUE) ;
    }
        /* Otherwise, this is a duplicate request.  We need to
        set a flag to indicate a dupicate has been detected,
        and return a value of FALSE to show that the syntax has
        been violated. */
    else
      
    {
       (* repeatFlagPtr) = TRUE ;
       return (FALSE) ;
    }
}
/* ######################################################### */
/*                         TakesArgs                         */
/* ######################################################### */
/*
     THIS FUNCTION TESTS WHETHER THE OPTION REPRESENTED BY chr
     IS ONE THAT CAN TAKE ONE OR MORE ARGUMENTS.
*/
int TakesArgs(chr)
     char chr ;
{
  int answer ;
  
  switch (chr)
  {
    case 'm':  answer = TRUE  ; break ;
    case 'p':  answer = TRUE  ; break ;    
    default :  answer = FALSE ; break ;
  }
  return (answer) ;
}
/* ######################################################### */
/*                        GetOptArgs                         */
/* ######################################################### */
/*
    THE JOB OF THIS FUNCTION IS TO FIGURE OUT WHICH OPTION
    REQUIRES THE ARGS, AND TO CALL A FUNCTION THAT IS A
    SPECIALIST AT GETTING THE ARGS FOR THAT OPTION.
*/
int GetOptArgs(argsLftPt, curArgPt, argv, cPt, mPt) 
       int * argsLftPt ;
       int * curArgPt ;
       char * argv[] ;
       commandType * cPt ;
       machineType * mPt ;
{
  int answer ;
  
  switch (argv[(* curArgPt)-1][1])
  {
    case 'm':  answer = mOptArgs(argsLftPt, curArgPt, argv, cPt, mPt) ;
               break ;
    case 'p':  answer = pOptArgs(argsLftPt, curArgPt, argv, cPt, mPt) ;
               break ;
    default :  answer = FALSE ; break ;
  }
  return (answer) ;
}
/* ######################################################### */
/*                      PrintCommand                         */
/* ######################################################### */
void PrintCommand (argc, argv, commandPtr) 
       int argc ;  char * argv[] ;
       commandType *  commandPtr ;
{
   printf ("\n" ) ;
   printf ("The TYPED COMMAND WAS: \n\n" ) ;
   { int i ; for (i=0;i<argc;i++) printf ("%s ", argv[i]) ;}
   printf ("\n\n");
   printf ("THE INTERPRETED COMMAND IS: \n\n");
   printf ("infileName        = %s\n",   commandPtr->infileName) ;
   printf ("infileNameLength  = %d\n",   commandPtr->infileNameLength) ;
   printf ("checking          = %d\n",   commandPtr->checking) ;
   printf ("debugging         = %d\n",   commandPtr->debugging) ;
   printf ("fine              = %d\n",   commandPtr->fine) ;
   printf ("mincover          = %d\n",   commandPtr->mincover) ;
   printf ("scaleInfo         = %d\n",   commandPtr->scaleInfo) ;
   printf ("terse             = %d\n",   commandPtr->terse) ;
   printf ("verbose           = %d\n",   commandPtr->verbose) ;
   printf ("placeBars         = %d\n",   commandPtr->placeBars) ;
   printf ("magnifier         = %.8g\n", commandPtr->magnifier) ;
   printf ("numCellSizes      = %lu\n",  commandPtr->numCellSizes) ;
   printf ("saturationPoint   = %lu\n",  commandPtr->saturationPoint) ;
   printf ("fineMeshPoint     = %lu\n",  commandPtr->fineMeshPoint) ;
   printf ("invalidOption     = %d\n",   commandPtr->invalidOption) ;
   printf ("repeatedOption    = %d\n",   commandPtr->repeatedOption) ;
   printf ("mutexOptions      = %d\n",   commandPtr->mutexOptions) ;
   printf ("\n") ;
   printf ("NOTE:\n") ;
   printf ("\n") ;
   printf ("If there was one or more errors on your command line,\n") ;
   printf ("default value(s) may appear above instead of value(s) you \n") ;
   printf ("tried to specify on the command line.\n") ;
   printf ("Also, if you used the -f option, the number of cell sizes\n") ;
   printf ("displayed here is only a base value.  FD will actually use\n") ;
   printf ("a multiple of this value.\n") ;
   printf ("\n") ;
   fflush(stdout);
   return ;
}
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */     
/*                        end getcom.c                       */
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */
